//
//  MTLUserInterfaceValidationUpdate.m
//  EnablerTest
//
//  Created by narita on 2013/01/28.
//  Copyright (c) 2013年 narita. All rights reserved.
//

#import "MTLUserInterfaceValidationUpdate.h"

@implementation MTLWindow

#pragma mark -
#pragma mark class methods

#pragma mark -
#pragma mark init methods

- (id) initWithContentRect:(NSRect)contentRect
                 styleMask:(NSUInteger)aStyle
                   backing:(NSBackingStoreType)bufferingType
                     defer:(BOOL)flag
{
    self = [ super initWithContentRect:contentRect
                             styleMask:aStyle
                               backing:bufferingType
                                 defer:flag];
    if( self )
    {
        self.autovalidates = YES;
    }
    
    return self;
}

#pragma mark -
#pragma mark dealloc

#pragma mark -
#pragma mark NSCopying, hash, isEqual:

#pragma mark -
#pragma mark NSCoding methods

#pragma mark -
#pragma mark awakeFromNib-like methods

#pragma mark -
#pragma mark action methods

#pragma mark -
#pragma mark delegate/datasource methods

#pragma mark -
#pragma mark accessor methods (in pairs)

@synthesize autovalidates;

#pragma mark -
#pragma mark Utility methods

- (void) update
{    
    if( self.autovalidates == YES && [[self contentView] isHiddenOrHasHiddenAncestor] == NO )
    {
        [[self contentView] updateSubvewsUserInterfaceValidation];
    }
    
    [super update];
}

@end

// ===========================================================================================
#pragma mark -

@implementation MTLPanel

#pragma mark -
#pragma mark class methods

#pragma mark -
#pragma mark init methods

- (id) initWithContentRect:(NSRect)contentRect
                 styleMask:(NSUInteger)aStyle
                   backing:(NSBackingStoreType)bufferingType
                     defer:(BOOL)flag
{
    self = [ super initWithContentRect:contentRect
                             styleMask:aStyle
                               backing:bufferingType
                                 defer:flag];
    if( self )
    {
        self.autovalidates = YES;
    }
    
    return self;
}

#pragma mark -
#pragma mark dealloc

#pragma mark -
#pragma mark NSCopying, hash, isEqual:

#pragma mark -
#pragma mark NSCoding methods

#pragma mark -
#pragma mark awakeFromNib-like methods

#pragma mark -
#pragma mark action methods

#pragma mark -
#pragma mark delegate/datasource methods

#pragma mark -
#pragma mark accessor methods (in pairs)

@synthesize autovalidates;

#pragma mark -
#pragma mark Utility methods

- (void) update
{
    if( self.autovalidates && [[self contentView] isHiddenOrHasHiddenAncestor] == NO )
    {
        [[self contentView] updateSubvewsUserInterfaceValidation];
    }
    
    [super update];
}

@end

// ===========================================================================================
#pragma mark -

@implementation NSView (MTLUserInterfaceValidationUpdate)

- (void) updateSubvewsUserInterfaceValidation
{
    // このviewの状態を更新する
    if( [self respondsToSelector:@selector(userInterfaceValidationUpdate)] )
    {
        [self userInterfaceValidationUpdate];
    }
    
    // 子viewのupdateSubvewsUserInterfaceValidationを再帰的に呼出して全てのviewの状態を更新する
    for(NSView* i in [self subviews])
    {
        if( [i isHiddenOrHasHiddenAncestor] == NO )
        {
            [i updateSubvewsUserInterfaceValidation];
        }
    }
}

@end

// ===========================================================================================
#pragma mark -

@implementation NSControl (MTLUserInterfaceValidationUpdate)

#pragma mark -
#pragma mark class methods

#pragma mark -
#pragma mark init methods

#pragma -
#pragma mark dealloc

#pragma mark -
#pragma mark NSCopying, hash, isEqual:

#pragma mark -
#pragma mark NSCoding methods

#pragma mark -
#pragma mark awakeFromNib-like methods

#pragma mark -
#pragma mark action & canAction methods

#pragma mark -
#pragma mark delegate/datasource methods

#pragma mark -
#pragma mark accessor methods (in pairs)

#pragma mark -
#pragma mark Utility methods

- (void) userInterfaceValidationUpdate
{    
	// actionが設定されているかをチェック。(target == nil)の場合はfirstResponderだから有効なのでtargetのnilチェックはしない。
    if( [self action] != nil && [self window] != nil && [self isHiddenOrHasHiddenAncestor] == NO)
	{
        BOOL                                  theResult    = NO;
		NSObject<NSUserInterfaceValidations>* theValidator = nil;
        
		// レスポンダーチェーンをたどって実際にactionを処理するオブジェクトを探す。
        // そのオブジェクトがメニューやボタンのenable/disableの値を決定出来るはず
		theValidator = [NSApp targetForAction:[self action]
										   to:[self target]
										 from:self];
		
        if ((theValidator == nil) || ![theValidator respondsToSelector:[self action]])
		{
			// actionを実行できるレスポンダーが無い場合は、ボタンは押せない
			theResult = NO;
		}
		else if ([theValidator respondsToSelector:@selector(validateUserInterfaceItem:)])
		{
            // actionを実行できるレスポンダーが"validateUserInterfaceItem:"を実装しているならば従う
            theResult = [theValidator validateUserInterfaceItem:(id<NSValidatedUserInterfaceItem>)self];
		}
		else
		{
			// レスポンダーがあるけど、"validateUserInterfaceItem"メソッドが無い場合は、ボタンは押せる。
            theResult = YES;
		}
        
        // enabledにCocoaBindingが設定されている場合は
        // CocoaBindingの設定を優先させるために設定しない
        if( [self infoForBinding:@"enabled"] == nil )
        {
            [self setEnabled:theResult];
        }
	}
}

@end

